@@ -1,3 +1,4 @@
+import copy
 from pathlib import Path
 from collections import defaultdict
 from lightwood.constants.lightwood import ColumnDataTypes
@@ -11,7 +12,7 @@ from lightwood.constants.lightwood import ColumnDataTypes
 from mindsdb_native.libs.constants.mindsdb import *
 from mindsdb_native.config import *
 from mindsdb_native.libs.helpers.stats_helpers import sample_data
-
+from mindsdb_native.libs.helpers.general_helpers import evaluate_accuracy
 
 class LightwoodBackend():
 
@@ -72,7 +73,7 @@ class LightwoodBackend():
 
         # Make all order column cells lists
         for group in ts_groups:
-            for order_col in order_by:
+            for order_col in order_by + self.transaction.lmd['tss']['historical_columns']:
                 for i in range(len(ts_groups[group])):
                     ts_groups[group][order_col].iloc[i] = [
                         ts_groups[group][order_col].iloc[i]
@@ -80,7 +81,7 @@ class LightwoodBackend():
 
         # Add previous rows
         for group in ts_groups:
-            for order_col in order_by:
+            for order_col in order_by + self.transaction.lmd['tss']['historical_columns']:
                 for i in range(len(ts_groups[group])):
                     previous_indexes = [*range(max(0, i - window), i)]
 
@@ -103,12 +104,20 @@ class LightwoodBackend():
                     previous_target_values = list(ts_groups[k][target_column])
                     del previous_target_values[-1]
                     previous_target_values = [None] + previous_target_values
-                    ts_groups[k]['previous_' + target_column] = previous_target_values
+
+                    previous_target_values_ts = []
+                    for i in range(len(previous_target_values)):
+                        arr = previous_target_values[max(i-window,0):i+1]
+                        while len(arr) <= window:
+                            arr = [None] + arr
+                        previous_target_values_ts.append(arr)
+
+                    ts_groups[k]['previous_' + target_column] = previous_target_values_ts
 
         combined_df = pd.concat(list(ts_groups.values()))
 
         if 'make_predictions' in combined_df.columns:
-            combined_df = pd.DataFrame(combined_df[combined_df['make_predictions']])
+            combined_df = pd.DataFrame(combined_df[combined_df['make_predictions'].astype(bool) == True])
             del combined_df['make_predictions']
 
         return combined_df, secondary_type_dict
@@ -181,15 +190,39 @@ class LightwoodBackend():
             col_config.update(other_keys)
 
             if col_name in self.transaction.lmd['predict_columns']:
+                if self.transaction.lmd['tss']['is_timeseries']:
+                    col_config['additional_info'] = {
+                        'nr_predictions': self.transaction.lmd['tss']['nr_predictions']
+                    }
                 config['output_features'].append(col_config)
+
+                if self.transaction.lmd['tss']['use_previous_target']:
+                    p_col_config = copy.deepcopy(col_config)
+                    p_col_config['name'] = f"previous_{p_col_config['name']}"
+                    p_col_config['type'] = ColumnDataTypes.TIME_SERIES
+
+                    if 'secondary_type' in col_config:
+                        p_col_config['secondary_type'] = col_config['secondary_type']
+
+                    config['input_features'].append(p_col_config)
+
             else:
+                if self.transaction.lmd['tss']['historical_columns']:
+                    if 'secondary_type' in col_config:
+                        col_config['secondary_type'] = col_config['secondary_type']
+                    col_config['type'] = ColumnDataTypes.TIME_SERIES
+
                 config['input_features'].append(col_config)
 
         config['data_source'] = {}
         config['data_source']['cache_transformed_data'] = not self.transaction.lmd['force_disable_cache']
 
-        config['mixer'] = {}
-        config['mixer']['selfaware'] = self.transaction.lmd['use_selfaware_model']
+        config['mixer'] = {
+            'class': lightwood.mixers.NnMixer,
+            'kwargs': {
+                'selfaware': self.transaction.lmd['use_selfaware_model']
+            }
+        }
 
         return config
 
@@ -245,33 +278,99 @@ class LightwoodBackend():
 
         lightwood_config = self._create_lightwood_config(secondary_type_dict)
 
-
-        self.predictor = lightwood.Predictor(lightwood_config)
-
-        # Evaluate less often for larger datasets and vice-versa
-        eval_every_x_epochs = int(round(1 * pow(10,6) * (1/len(train_df))))
-
-        # Within some limits
-        if eval_every_x_epochs > 200:
-            eval_every_x_epochs = 200
-        if eval_every_x_epochs < 3:
-            eval_every_x_epochs = 3
-
-        logging.getLogger().setLevel(logging.DEBUG)
-        if self.transaction.lmd['stop_training_in_x_seconds'] is None:
-            self.predictor.learn(from_data=train_df, test_data=test_df, callback_on_iter=self.callback_on_iter, eval_every_x_epochs=eval_every_x_epochs)
-        else:
-            self.predictor.learn(from_data=train_df, test_data=test_df, stop_training_after_seconds=self.transaction.lmd['stop_training_in_x_seconds'], callback_on_iter=self.callback_on_iter, eval_every_x_epochs=eval_every_x_epochs)
-
-        self.transaction.log.info('Training accuracy of: {}'.format(self.predictor.train_accuracy))
+        lightwood_train_ds = lightwood.api.data_source.DataSource(
+            train_df,
+            config=lightwood_config
+        )
+        lightwood_test_ds = lightwood_train_ds.make_child(test_df)
 
         self.transaction.lmd['lightwood_data']['save_path'] = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, self.transaction.lmd['name'], 'lightwood_data')
         Path(CONFIG.MINDSDB_STORAGE_PATH).joinpath(self.transaction.lmd['name']).mkdir(mode=0o777, exist_ok=True, parents=True)
+
+        logging.getLogger().setLevel(logging.DEBUG)
+
+        predictors_and_accuracies = []
+
+        use_mixers = self.transaction.lmd.get('use_mixers', None)
+        if use_mixers is not None:
+            if isinstance(use_mixers, list):
+                mixer_classes = use_mixers
+            else:
+                mixer_classes = [use_mixers]
+        else:
+            mixer_classes = lightwood.mixers.BaseMixer.__subclasses__()
+
+        for mixer_class in mixer_classes:
+            lightwood_config['mixer']['kwargs'] = {}
+            lightwood_config['mixer']['class'] = mixer_class
+
+            if lightwood_config['mixer']['class'] == lightwood.mixers.NnMixer:
+                # Evaluate less often for larger datasets and vice-versa
+                eval_every_x_epochs = int(round(1 * pow(10, 6) * (1 / len(train_df))))
+
+                # Within some limits
+                if eval_every_x_epochs > 200:
+                    eval_every_x_epochs = 200
+                if eval_every_x_epochs < 3:
+                    eval_every_x_epochs = 3
+
+                kwargs = lightwood_config['mixer']['kwargs']
+
+                kwargs['callback_on_iter'] = self.callback_on_iter
+                kwargs['eval_every_x_epochs'] = eval_every_x_epochs / len(mixer_classes)
+                kwargs['stop_training_after_seconds'] = self.transaction.lmd['stop_training_in_x_seconds']
+
+            self.predictor = lightwood.Predictor(lightwood_config.copy())
+
+            self.predictor.learn(
+                from_data=lightwood_train_ds,
+                test_data=lightwood_test_ds
+            )
+
+            self.transaction.log.info('[{}] Training accuracy of: {}'.format(
+                mixer_class.__name__,
+                self.predictor.train_accuracy
+            ))
+
+            validation_predictions = self.predict('validate')
+            validation_accuracy = evaluate_accuracy(
+                validation_predictions,
+                self.transaction.input_data.validation_df,
+                self.transaction.lmd['stats_v2'],
+                self.transaction.lmd['predict_columns'],
+                backend=self
+            )
+
+            predictors_and_accuracies.append((
+                self.predictor,
+                validation_accuracy
+            ))
+
+        best_predictor, best_accuracy = max(predictors_and_accuracies, key=lambda x: x[1])
+
+        # Find predictor with NnMixer
+        for predictor, accuracy in predictors_and_accuracies:
+            if isinstance(predictor._mixer, lightwood.mixers.NnMixer):
+                nn_mixer_predictor, nn_mixer_predictor_accuracy = predictor, accuracy
+                break
+        else:
+            nn_mixer_predictor, nn_mixer_predictor_accuracy = None, None
+
+        self.predictor = best_predictor
+
+        # If difference between accuracies of best predictor and NnMixer predictor
+        # is small, then use NnMixer predictor
+        if nn_mixer_predictor is not None:
+            SMALL_ACCURACY_DIFFERENCE = 0.01
+            if (best_accuracy - nn_mixer_predictor_accuracy) < SMALL_ACCURACY_DIFFERENCE:
+                self.predictor = nn_mixer_predictor
+
         self.predictor.save(path_to=self.transaction.lmd['lightwood_data']['save_path'])
 
-    def predict(self, mode='predict', ignore_columns=None):
+    def predict(self, mode='predict', ignore_columns=None, all_mixers=False):
         if ignore_columns is None:
             ignore_columns = []
+
         if self.transaction.lmd['use_gpu'] is not None:
             lightwood.config.config.CONFIG.USE_CUDA = self.transaction.lmd['use_gpu']
 
