@@ -1,9 +1,11 @@
 import os
+from pathlib import Path
 import pickle
 import shutil
 import zipfile
 import traceback
 import uuid
+import tempfile
 
 from mindsdb_native.config import CONFIG
 from mindsdb_native.__about__ import __version__
@@ -11,6 +13,7 @@ from mindsdb_native.libs.data_types.mindsdb_logger import log
 from mindsdb_native.libs.controllers.transaction import AnalyseTransaction
 from mindsdb_native.libs.controllers.predictor import _get_memory_optimizations, _prepare_sample_settings, Predictor
 from mindsdb_native.libs.helpers.multi_data_source import getDS
+from mindsdb_native.libs.helpers.general_helpers import load_lmd, load_hmd
 from mindsdb_native.libs.constants.mindsdb import (MODEL_STATUS_TRAINED,
                                                    MODEL_STATUS_ERROR,
                                                    TRANSACTION_ANALYSE)
@@ -19,16 +22,16 @@ from mindsdb_native.libs.helpers.locking import MDBLock
 def validate(to_predict, from_data, accuracy_score_functions, learn_args=None, test_args=None):
             if learn_args is None: learn_args = {}
             if test_args is None: test_args = {}
-            
+
             name = str(uuid.uuid4()).replace('-','')
             predictor = Predictor(name)
-            
+
             predictor.learn(to_predict, from_data, **learn_args)
             validation_data = predictor.transaction.input_data.validation_df
 
             accuracy = predictor.test(when_data=validation_data, accuracy_score_functions=accuracy_score_functions, **test_args)
 
-            delete_model(name) 
+            delete_model(name)
 
             return accuracy
 
@@ -40,7 +43,7 @@ def cross_validate(to_predict, from_data, accuracy_score_functions, k=5, learn_a
 
         Alternatively we can just add train/test/valid split as advanced args and forgo the data splitter if they are specified (maybe have them be indexes so that the rest of the phases up to the splitter and run normally) + Do the splitting inside this function.
 
-        Same problem with timeseries argument support though. 
+        Same problem with timeseries argument support though.
     '''
     raise NotImplementedError('Cross validation is not implemented yet')
 
@@ -70,9 +73,7 @@ def analyse_dataset(from_data, sample_settings=None):
         model_columns_map = from_ds._col_map,
         type = transaction_type,
         sample_settings = sample_settings,
-        model_is_time_series = False,
-        model_group_by = [],
-        model_order_by = [],
+        tss={'is_timeseries':False},
         columns_to_ignore = [],
         data_preparation = {},
         predict_columns = [],
@@ -116,16 +117,15 @@ def export_predictor(model_name):
     with MDBLock('shared', 'predict_' + model_name):
         storage_file = model_name + '.zip'
         with zipfile.ZipFile(storage_file, 'w') as zip_fp:
-            for file_name in [model_name + '_heavy_model_metadata.pickle',
-                            model_name + '_light_model_metadata.pickle',
-                            model_name + '_lightwood_data']:
-                full_path = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, file_name)
+            for file_name in ['heavy_model_metadata.pickle',
+                              'light_model_metadata.pickle',
+                              'lightwood_data']:
+                full_path = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name, file_name)
                 zip_fp.write(full_path, os.path.basename(full_path))
 
             # If the backend is ludwig, save the ludwig files
             try:
-                ludwig_model_path = os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                                                model_name + '_ludwig_data')
+                ludwig_model_path = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name, 'ludwig_data')
                 for root, dirs, files in os.walk(ludwig_model_path):
                     for file in files:
                         full_path = os.path.join(root, file)
@@ -153,25 +153,18 @@ def rename_model(old_model_name, new_model_name):
             return True
 
         moved_a_backend = False
-        for extension in ['_lightwood_data', '_ludwig_data']:
-            shutil.move(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                                    old_model_name + extension),
-                        os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                                    new_model_name + extension))
+        for extension in ['lightwood_data', 'ludwig_data']:
+            shutil.move(
+                os.path.join(CONFIG.MINDSDB_STORAGE_PATH, old_model_name, extension),
+                os.path.join(CONFIG.MINDSDB_STORAGE_PATH, new_model_name, extension)
+            )
             moved_a_backend = True
 
         if not moved_a_backend:
             return False
 
-        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            old_model_name + '_light_model_metadata.pickle'),
-                'rb') as fp:
-            lmd = pickle.load(fp)
-
-        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            old_model_name + '_heavy_model_metadata.pickle'),
-                'rb') as fp:
-            hmd = pickle.load(fp)
+        lmd = load_lmd(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, old_model_name, 'light_model_metadata.pickle'))
+        hmd = load_lmd(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, old_model_name, 'heavy_model_metadata.pickle'))
 
         lmd['name'] = new_model_name
         hmd['name'] = new_model_name
@@ -195,19 +188,19 @@ def rename_model(old_model_name, new_model_name):
             return False
 
         with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            new_model_name + '_light_model_metadata.pickle'),
+                            new_model_name, 'light_model_metadata.pickle'),
                 'wb') as fp:
             pickle.dump(lmd, fp, protocol=pickle.HIGHEST_PROTOCOL)
 
         with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            new_model_name + '_heavy_model_metadata.pickle'),
+                            new_model_name, 'heavy_model_metadata.pickle'),
                 'wb') as fp:
             pickle.dump(hmd, fp, protocol=pickle.HIGHEST_PROTOCOL)
 
         os.remove(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            old_model_name + '_light_model_metadata.pickle'))
+                            old_model_name, 'light_model_metadata.pickle'))
         os.remove(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,
-                            old_model_name + '_heavy_model_metadata.pickle'))
+                            old_model_name, 'heavy_model_metadata.pickle'))
         return True
 
 
@@ -219,57 +212,60 @@ def delete_model(model_name):
     :return: bool (True/False) True if model was deleted
     """
     with MDBLock('exclusive', 'delete_' + model_name):
-        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name + '_light_model_metadata.pickle'), 'rb') as fp:
-            lmd = pickle.load(fp)
+        lmd = load_lmd(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name, 'light_model_metadata.pickle'))
 
-            try:
-                os.remove(lmd['lightwood_data']['save_path'])
-            except Exception:
-                pass
+        try:
+            os.remove(lmd['lightwood_data']['save_path'])
+        except Exception:
+            pass
 
-            try:
-                shutil.rmtree(lmd['ludwig_data']['ludwig_save_path'])
-            except Exception:
-                pass
+        try:
+            shutil.rmtree(lmd['ludwig_data']['ludwig_save_path'])
+        except Exception:
+            pass
 
-        for file_name in [model_name + '_heavy_model_metadata.pickle',
-                        model_name + '_light_model_metadata.pickle']:
-            os.remove(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, file_name))
+        shutil.rmtree(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name))
 
 
-def _import_model_by_name(model_name):
-    with MDBLock('exclusive', 'detele_' + model_name):
-        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name + '_light_model_metadata.pickle'), 'rb') as fp:
-            lmd = pickle.load(fp)
-
-        if 'ludwig_data' in lmd and 'ludwig_save_path' in lmd['ludwig_data']:
-            lmd['ludwig_data']['ludwig_save_path'] = str(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,os.path.basename(lmd['ludwig_data']['ludwig_save_path'])))
-
-        if 'lightwood_data' in lmd and 'save_path' in lmd['lightwood_data']:
-            lmd['lightwood_data']['save_path'] = str(os.path.join(CONFIG.MINDSDB_STORAGE_PATH,os.path.basename(lmd['lightwood_data']['save_path'])))
-
-        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name + '_light_model_metadata.pickle'), 'wb') as fp:
-            pickle.dump(lmd, fp,protocol=pickle.HIGHEST_PROTOCOL)
-
-
-def import_model(model_archive_path):
+def import_model(model_archive_path, new_name=None):
     """
     Import a mindsdb instance storage from an archive file.
 
     :param mindsdb_storage_dir: full_path that contains your mindsdb predictor zip file
     """
-    previous_models = os.listdir(CONFIG.MINDSDB_STORAGE_PATH)
-    shutil.unpack_archive(model_archive_path, extract_dir=CONFIG.MINDSDB_STORAGE_PATH)
+    previous_models = [str(p.name) for p in Path(CONFIG.MINDSDB_STORAGE_PATH).iterdir() if p.is_dir()]
+    extract_dir = tempfile.mkdtemp(dir=CONFIG.MINDSDB_STORAGE_PATH)
+    shutil.unpack_archive(model_archive_path, extract_dir=extract_dir)
 
-    new_model_files = set(os.listdir(CONFIG.MINDSDB_STORAGE_PATH)) - set(previous_models)
-    model_names = []
-    for file in new_model_files:
-        if '_light_model_metadata.pickle' in file:
-            model_name = file.replace('_light_model_metadata.pickle', '')
-            model_names.append(model_name)
+    try:
+        lmd = load_lmd(os.path.join(extract_dir, 'light_model_metadata.pickle'))
+    except Exception:
+        shutil.rmtree(extract_dir)
+        raise
 
-    for model_name in model_names:
-        _import_model_by_name(model_name)
+    if new_name is not None:
+        lmd['name'] = new_name
+    elif lmd['name'] is None:
+        lmd['name'] = extract_dir
+    
+    if lmd['name'] in previous_models:
+        shutil.rmtree(extract_dir)
+        raise Exception(f"Model with name '{lmd['name']}' already exists.")
+
+    shutil.move(
+        extract_dir,
+        os.path.join(CONFIG.MINDSDB_STORAGE_PATH, lmd['name'])
+    )
+
+    with MDBLock('exclusive', 'detele_' + lmd['name']):
+        if 'ludwig_data' in lmd and 'ludwig_save_path' in lmd['ludwig_data']:
+            lmd['ludwig_data']['ludwig_save_path'] = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, lmd['name'], 'ludwig_data')
+
+        if 'lightwood_data' in lmd and 'save_path' in lmd['lightwood_data']:
+            lmd['lightwood_data']['save_path'] = os.path.join(CONFIG.MINDSDB_STORAGE_PATH, lmd['name'], 'lightwood_data')
+
+        with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, lmd['name'], 'light_model_metadata.pickle'), 'wb') as fp:
+            pickle.dump(lmd, fp,protocol=pickle.HIGHEST_PROTOCOL)
 
     print('Model files loaded')
 
@@ -326,12 +322,18 @@ def get_model_data(model_name=None, lmd=None):
         pass
     elif model_name is not None:
         with MDBLock('shared', 'get_data_' + model_name):
-            with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, f'{model_name}_light_model_metadata.pickle'), 'rb') as fp:
-                lmd = pickle.load(fp)
+            lmd = load_lmd(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, model_name, 'light_model_metadata.pickle'))
 
     # ADAPTOR CODE
     amd = {}
 
+    if 'tss' in lmd:
+        if lmd['tss']['is_timeseries']:
+            amd['timeseries'] = {}
+            amd['timeseries']['user_settings'] = lmd['tss']
+        else:
+            amd['timeseries'] = None
+
     if 'stats_v2' in lmd:
         amd['data_analysis_v2'] = lmd['stats_v2']
 
@@ -401,8 +403,6 @@ def get_model_data(model_name=None, lmd=None):
             else:
                 accuracy_samples = None
 
-
-
             # Model analysis building for each of the predict columns
             mao = {
                 'column_name': col
@@ -474,7 +474,7 @@ def get_model_data(model_name=None, lmd=None):
             amd['model_analysis'].append(mao)
         else:
             if 'column_importances' in lmd and lmd['column_importances'] is not None:
-                if col not in [x[0] for x in lmd['model_order_by']]:
+                if not lmd['tss']['is_timeseries'] or col not in lmd['tss']['order_by']:
                     icm['importance_score'] = lmd['column_importances'][col]
             amd['data_analysis']['input_columns_metadata'].append(icm)
 
@@ -483,22 +483,21 @@ def get_model_data(model_name=None, lmd=None):
 
 def get_models():
     models = []
-    for fn in os.listdir(CONFIG.MINDSDB_STORAGE_PATH):
-        if '_light_model_metadata.pickle' in fn:
-            model_name = fn.replace('_light_model_metadata.pickle','')
-            try:
-                amd = get_model_data(model_name)
-                model = {}
-                for k in ['name', 'version', 'is_active', 'data_source', 'predict',
-                'status', 'train_end_at', 'updated_at', 'created_at','current_phase', 'accuracy']:
-                    if k in amd:
-                        model[k] = amd[k]
-                    else:
-                        model[k] = None
+    for p in [x for x in Path(CONFIG.MINDSDB_STORAGE_PATH).iterdir() if x.is_dir()]:
+        model_name = p.name
+        try:
+            amd = get_model_data(model_name)
+            model = {}
+            for k in ['name', 'version', 'is_active', 'data_source', 'predict',
+            'status', 'train_end_at', 'updated_at', 'created_at','current_phase', 'accuracy']:
+                if k in amd:
+                    model[k] = amd[k]
+                else:
+                    model[k] = None
 
-                models.append(model)
-            except Exception as e:
-                print(f"Can't adapt metadata for model: '{model_name}' when calling `get_models()`")
-                raise
+            models.append(model)
+        except Exception:
+            print(f"Can't adapt metadata for model: '{model_name}' when calling `get_models()`")
+            raise
 
     return models
